/*
    This file is part of Accountant.

    Accountant is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Accountant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with Accountant.  If not, see <http://www.gnu.org/licenses/>.
 */
accountantApp

.factory("ScheduledOperations", ["$resource", function($resource) {
    return $resource(
        "/api/scheduled_operations/:id", {
            id: "@id"
        }
    );
}])

.controller(
    "SchedulerController", [
        "$scope", "$http", "$rootScope", "$filter", "$routeParams", "ScheduledOperations",
        function($scope, $http, $rootScope, $filter, $routeParams, ScheduledOperations) {
    // Operations store and selection
    $scope.operations = [];
    $scope.selectedOperation = null;

    // Placeholder for saved value to cancel entry edition
    $scope.savedOperation = null;

    $scope.categories = [];

    $scope.resetNewOperation = function() {
        $scope.newOperation = new ScheduledOperations({});
    };

    $scope.resetNewOperation();

    $scope.loadOperations = function(accountId) {
      // Clean up selected entry.
      $scope.selectedOperation = null;
      $scope.savedOperation = null;

      $scope.operations = ScheduledOperations.query({
          account: $routeParams.accountId
      }, function(data) {
        $scope.$emit("operationsLoadedEvent", {operations: data});
      });
    };

    // Returns true if the entry is in editing state.
    $scope.isEditing = function(operation) {
      return operation.state === 'edit';
    };

    $scope.isDisplaying = function(operation) {
      return operation.id && (!operation.state || operation.state === 'display');
    };

    $scope.createOperation = function(operation) {
        operation.account_id = $routeParams.accountId;

        operation.$save(function(data) {
            $scope.resetNewOperation();

            $scope.$emit("operationCreatedEvent", data);
        });
    };

    $scope.$on("operationCreatedEvent", function(e, operation) {
        new PNotify({
            type: "success",
            title: "Save",
            text: "Operation #" + operation.id + " created."
        });
    });

    $scope.saveOperation = function(operation) {
      operation.$save(function(data) {
        $scope.$emit("operationSavedEvent", operation);
      });
    };

    $scope.$on("operationSavedEvent", function(e, operation) {
        new PNotify({
            type: "success",
            title: "Save",
            text: "Operation #" + operation.id + " saved."
        });
    });

    $scope.editOperation = function(operation) {
        // Enter edit state.
        operation.state='edit';
    };

    $scope.cancelEditOperation = function(operation) {
        operation.$get();
    };

    $scope.removeOperation = function(operation, modalScope) {
      // Cancel current editing.
      if (!$scope.isNew(operation)) {
        $http.delete("/api/scheduled_operations/" + operation.id).success(function (result) {
          $.pnotify({
              type: "success",
              title: "Delete",
              text: result
          });

          // Send the "entry removed" event.
          $scope.$emit("operationRemovedEvent", operation);

          $scope.closeModal(modalScope);
        }).error(function (data) {
          $.pnotify({
              type: "error",
              title: "Delete",
              text: data
          });

          $scope.closeModal(modalScope);
        });
      }
    };

    $scope.closeModal = function(modalScope) {
      // Close the modal dialog
      if(modalScope && modalScope.dismiss) {
        modalScope.dismiss();
      }
    };

    $rootScope.$on("accountSelectedEvent", function(event, args){
        $scope.loadOperations(args.account.id);
    });

    $scope.loadOperations($routeParams.accountId);
}]);
